@@ -49,9 +49,10 @@ class IdempotentSessionWizardView(SessionWizardView):
         if step is None:
             step = self.steps.current
         form_list = self.get_form_list()
-        key = form_list.keyOrder.index(step) - 1
+        keys = list(form_list.keys())
+        key = keys.index(step) - 1
         if key >= 0:
-            for prev_step in form_list.keyOrder[key::-1]:
+            for prev_step in keys[key::-1]:
                 if self.is_step_visible(prev_step):
                     return prev_step
         return None
@@ -65,12 +66,23 @@ class IdempotentSessionWizardView(SessionWizardView):
         if step is None:
             step = self.steps.current
         form_list = self.get_form_list()
-        key = form_list.keyOrder.index(step) + 1
-        for next_step in form_list.keyOrder[key:]:
+        keys = list(form_list.keys())
+        key = keys.index(step) + 1
+        for next_step in keys[key:]:
             if self.is_step_visible(next_step):
                 return next_step
         return None
 
+    def post(self, *args, **kwargs):
+        """
+        Check if the current step is still available. It might not be if
+        conditions have changed.
+        """
+        if self.steps.current not in self.steps.all:
+            return self.render_goto_step(self.steps.all[-1])
+
+        return super(IdempotentSessionWizardView, self).post(*args, **kwargs)
+
     def process_step(self, form):
         """
         Stores the validated data for `form` and cleans out validated forms
@@ -88,10 +100,11 @@ class IdempotentSessionWizardView(SessionWizardView):
         # It is assumed that earlier steps affect later steps; so even though
         # those forms might not be idempotent, we'll remove the validated data
         # to force re-entry.
-        #form_list = self.get_form_list(idempotent=False)
+        # form_list = self.get_form_list(idempotent=False)
         form_list = self.get_form_list()
-        key = form_list.keyOrder.index(step) + 1
-        for next_step in form_list.keyOrder[key:]:
+        keys = list(form_list.keys())
+        key = keys.index(step) + 1
+        for next_step in keys[key:]:
             self.storage.validated_step_data.pop(next_step, None)
 
         return super(IdempotentSessionWizardView, self).process_step(form)
